@@ -1,40 +1,80 @@
-# $Id: Cache.pm,v 1.15 2000/10/02 17:35:34 matt Exp $
+# $Id: Cache.pm,v 1.3 2002/02/18 18:36:20 darobin Exp $
 
 package Apache::AxKit::Cache;
 use strict;
 
 use Apache;
+use Apache::Constants qw(OK DECLINED);
 use Apache::AxKit::Exception;
 use Digest::MD5 ();
 use Compress::Zlib;
-use Fcntl qw(:DEFAULT);
+use Fcntl qw(:flock O_RDWR O_WRONLY O_CREAT O_RDONLY);
 
 # use vars qw/$COUNT/;
 
 sub new {
     my $class = shift;
     my ($r, $xmlfile, @extras) = @_;
-    my $key = Digest::MD5->new->add("$xmlfile:" . join(':', @extras))->hexdigest;
+    
+    my $gzip = 0;
+    if ($xmlfile =~ /\.gzip/) {
+        $gzip++;
+#        @extras = grep(!/gzip/, @extras);
+    }
+    
+    my $key = Digest::MD5->new->add(
+            join(':', 
+                $r->get_server_name,
+                $r->get_server_port,
+                $xmlfile,
+                @extras
+            ))->hexdigest;
+    
+    AxKit::Debug(7, "Cache: key = $key");
+
+    my $primary = substr($key,0,2,'');
+    my $secondary = substr($key,0,2,'');
     
 #    warn "New for: $xmlfile:" . join(':', @extras). "\n";
     
     my $cachedir = $AxKit::Cfg->CacheDir();
     
     my $no_cache;
-    
-    if (!-e $cachedir) {
-        if (!mkdir($cachedir, 0777)) {
-            warn "Can't create cache directory '$cachedir': $!\n";
-            $no_cache = 1;
-        }
+
+    if ($AxKit::Cfg->NoCache()) {
+        $no_cache = 1;
     }
     
+    if (!$no_cache) {
+       if (!-e $cachedir) {
+           if (!mkdir($cachedir, 0777)) {
+               AxKit::Debug(2, "Can't create cache directory '$cachedir': $!");
+               $no_cache = 1;
+           }
+       }
+
+       if (!-e "$cachedir/$primary") {
+           if (!mkdir("$cachedir/$primary", 0777)) {
+               AxKit::Debug(1, "Can't create cache directory '$cachedir/$primary': $!");
+               $no_cache = 1;
+           }
+       }
+       
+       if (!-e "$cachedir/$primary/$secondary") {
+           if (!mkdir("$cachedir/$primary/$secondary", 0777)) {
+               AxKit::Debug(1, "Can't create cache directory '$cachedir/$primary/$secondary': $!");
+               $no_cache = 1;
+           }
+       }
+   }
+
     my $self = bless { 
         apache => $r,
         key => $key, 
         no_cache => $no_cache, 
         dir => $cachedir,
-        file => "$cachedir/$key",
+        file => "$cachedir/$primary/$secondary/$key",
+        gzip => $gzip,
 #        extras => \@extras,
         }, $class;
 
@@ -47,6 +87,18 @@ sub new {
     return $self;
 }
 
+sub _get_stats {
+    my $self = shift;
+    return if $self->{mtime};
+    my @stats = stat($self->{file});
+    my $exists = -e _ && -r _;
+    if ($exists and $self->{gzip}) {
+        $exists = -e $self->{file} . '.gz' and -r _;
+    }
+    $self->{file_exists} = $exists;
+    $self->{mtime} = $stats[9];
+}
+
 # sub DESTROY {
 #     AxKit::Debug(7, "Cache->DESTROY Count: ".--$COUNT);
 # }
@@ -54,25 +106,27 @@ sub new {
 sub write {
     my $self = shift;
     return if $self->{no_cache};
+    AxKit::Debug(7, "[Cache] writing cache file $self->{file}");
     my $fh = Apache->gensym();
-    if (sysopen($fh, $self->{file}.'new', O_RDWR|O_CREAT)) {
-        flock($fh, 2);
-        seek($fh, 0, 0);
-        truncate($fh, 0);
+    my $tmp_filename = $self->{file}."new$$";
+    if (sysopen($fh, $tmp_filename, O_WRONLY|O_CREAT)) {
+        # flock($fh, LOCK_EX);
+        # seek($fh, 0, 0);
+        # truncate($fh, 0);
         print $fh $_[0];
         close $fh;
-        rename($self->{file}.'new', $self->{file}) 
+        rename($tmp_filename, $self->{file}) 
                 || throw Apache::AxKit::Exception::IO( -text => "Couldn't rename cachefile: $!");
     }
     else {
         throw Apache::AxKit::Exception::IO( -text => "Couldn't open cachefile for writing: $!");
     }
     
-    if ($AxKit::Cfg->GzipOutput) {
+    if ($self->{gzip} && $AxKit::Cfg->GzipOutput) {
         AxKit::Debug(3, 'Creating gzip output cache');
         my $fh = Apache->gensym();
         if (sysopen($fh, $self->{file}.'new.gz', O_RDWR|O_CREAT)) {
-            flock($fh, 2);
+            flock($fh, LOCK_EX);
             seek($fh, 0, 0);
             truncate($fh, 0);
             print $fh ''.Compress::Zlib::memGzip($_[0]);
@@ -91,10 +145,11 @@ sub read {
     return if $self->{no_cache};
     my $fh = Apache->gensym();
     if (sysopen($fh, $self->{file}, O_RDONLY)) {
-        flock($fh, 1);
+        flock($fh, LOCK_SH);
         local $/;
         return <$fh>;
-        # closes and unlocks automatically upon loss of scope
+        # close($fh);
+        # close unlocks automatically
     }
     return '';
 }
@@ -104,7 +159,7 @@ sub get_fh {
     return if $self->{no_cache};
     my $fh = Apache->gensym();
     if (sysopen($fh, $self->{file}, O_RDONLY)) {
-        flock($fh, 1);
+        flock($fh, LOCK_SH);
         return $fh;
     }
     else {
@@ -112,6 +167,39 @@ sub get_fh {
     }
 }
 
+sub set_type {
+    my $self = shift;
+    return if $self->{no_cache};
+    
+    my $fh = Apache->gensym();
+    if (sysopen($fh, $self->{file}.'newtype', O_RDWR|O_CREAT)) {
+        flock($fh, LOCK_EX);
+        seek($fh, 0, 0);
+        truncate($fh, 0);
+        print $fh $_[0];
+        close $fh;
+        rename($self->{file}.'newtype', $self->{file}.'.type') 
+                || throw Apache::AxKit::Exception::IO( -text => "Couldn't rename type cachefile: $!");
+    }
+    else {
+        throw Apache::AxKit::Exception::IO( -text => "Couldn't open type cachefile for writing: $!");
+    }
+}
+
+sub get_type {
+    my $self = shift;
+    return if $self->{no_cache};
+    my $fh = Apache->gensym();
+    if (sysopen($fh, $self->{file}.'.type', O_RDONLY)) {
+        flock($fh, LOCK_SH);
+        local $/;
+        return <$fh>;
+        # close($fh);
+        # close unlocks automatically
+    }
+    return '';
+}
+
 sub deliver {
     my $self = shift;
     return if $self->{no_cache};
@@ -120,24 +208,37 @@ sub deliver {
     {
         # get content-type
         AxKit::Debug(4, "Cache: Getting content-type");
-        my $typecache = Apache::AxKit::Cache->new($r, $self->{key} . '.type');
-        if (my $type = $typecache->read()) {
+        if (my $type = $self->get_type) {
+            AxKit::Debug(4, "Cache: setting content-type: $type");
             $r->content_type($type);
         }
     }
     
+    if ($r->content_type eq 'changeme' && !$r->notes('axkit_passthru_type')) {
+        $AxKit::Cfg->AllowOutputCharset(1);
+        $r->content_type('text/html; charset=' . ($AxKit::Cfg->OutputCharset || "UTF-8"));
+    }
+    elsif ($r->notes('axkit_passthru_type')) {
+        $r->content_type($AxKit::OrigType);
+    }
+
+    
     my ($transformer, $doit) = AxKit::get_output_transformer();
     
     if ($doit) {
         AxKit::Debug(4, "Cache: Transforming content and printing to browser");
         $r->send_http_header() unless lc($r->dir_config('Filter')) eq 'on';
         $r->print( $transformer->( $self->read() ) );
-        throw Apache::AxKit::Exception::OK();
+        return OK;
     }
     else {
         AxKit::Debug(4, "Cache: Sending untransformed content to browser");
 
-        if ($AxKit::Cfg->DoGzip) {
+        # Make sure we unset PATH_INFO or wierd things can happen!
+        $ENV{PATH_INFO} = '';
+        $r->path_info('');
+        
+        if ($self->{gzip} && $AxKit::Cfg->DoGzip) {
             AxKit::Debug(4, 'Cache: Delivering gzipped output');
             $r->filename($self->{file}.'.gz');
         }
@@ -145,9 +246,7 @@ sub deliver {
             $r->filename($self->{file});
         }
         
-        throw Apache::AxKit::Exception::Declined(
-                reason => "delivering cached copy"
-                );
+        return DECLINED;
     }
     
 }
@@ -159,13 +258,23 @@ sub reset {
 
 sub mtime {
     my $self = shift;
-    return -M $self->{file};
+    $self->_get_stats;
+    return $self->{mtime} if exists $self->{mtime};
+    return ($self->{mtime} = (stat($self->{file}))[9]);
+}
+
+sub has_changed {
+    my $self = shift;
+    my $time = shift;
+    return $self->mtime > $time;
 }
 
 sub exists {
     my $self = shift;
     return if $self->{no_cache};
-    return -e $self->{file};
+    $self->_get_stats;
+    return $self->{file_exists} if exists $self->{file_exists};
+    return ($self->{file_exists} = -e $self->{file});
 }
 
 sub key {
@@ -181,17 +290,6 @@ sub no_cache {
     if ($_[0]) {
         AxKit::Debug(8, "Turning off cache!");
         $self->{no_cache} = 1;
-#         my $r = $self->{apache};
-#         my $fh = Apache->gensym();
-#         if (sysopen($fh, $self->{file}, O_RDONLY)) {
-#             flock($fh, 1);
-#             $r->send_http_header();
-#             while (<$fh>) {
-#                 $r->print($_);
-#             }
-#             close $fh;
-#         }
-        
         $self->reset();
     }
     
