@@ -1,29 +1,30 @@
-# $Id: ConfigReader.pm,v 1.17 2000/10/01 22:08:40 matt Exp $
+# $Id: ConfigReader.pm,v 1.2 2002/04/02 16:27:54 matts Exp $
 
 package Apache::AxKit::ConfigReader;
 
 use strict;
 
-use Apache::ModuleConfig ();
-
 # use vars qw/$COUNT/;
 
 sub new {
     my $class = shift;
     my $r = shift;
     
-    my $cfg = Apache::ModuleConfig->get($r, 'AxKit') || {};
+    my $cfg = AxKit::get_config($r) || {};
+    
+#    use Apache::Peek 'Dump';
+#    Dump($cfg);
+#     use Data::Dumper;
+#     $Data::Dumper::Indent = 1;
+#     warn("Cfg: ", Data::Dumper->Dump([$cfg], ['cfg']));
+
+    my $self = bless { apache => $r, cfg => $cfg, output_charset_ok => 0 }, $class;
     
     if (my $alternate = $cfg->{ConfigReader} || $r->dir_config('AxConfigReader')) {
-        $class = $alternate;
-        my $pkg = $class;
-        $pkg =~ s/::/\//g;
-        require "$pkg.pm";
+        AxKit::reconsecrate($self, $alternate);
     }
     
-#     AxKit::Debug(7, "ConfigReader->new count: ".++$COUNT);
-    
-    return bless { apache => $r, cfg => $cfg }, $class;
+    return $self;
 }
 
 # sub DESTROY {
@@ -66,6 +67,27 @@ sub ProviderClass {
     return 'Apache::AxKit::Provider::File';
 }
 
+sub DependencyChecks {
+    my $self = shift;
+    return $self->{cfg}{DependencyChecks};
+}
+
+sub StyleProviderClass {
+    my $self = shift;
+    if (my $alternate = $self->{cfg}{StyleProvider} || 
+            $self->{apache}->dir_config('AxStyleProvider')) {
+        return $alternate;
+    }
+    
+    return 'Apache::AxKit::Provider::File';
+}
+
+sub NoCache {
+    my $self = shift;
+    return $self->{cfg}{NoCache} || 
+            $self->{apache}->dir_config('AxNoCache');
+}
+
 sub PreferredStyle {
     my $self = shift;
     
@@ -105,14 +127,68 @@ sub DebugLevel {
             0;
 }
 
+sub DebugTime {
+    my $self = shift;
+    return $self->{apache}->dir_config('AxDebugTime') || 0;
+}
+
+sub StackTrace {
+    my $self = shift;
+    return $self->{cfg}{StackTrace} ||
+            $self->{apache}->dir_config('AxStackTrace') ||
+            0;
+}
+
+sub LogDeclines {
+    my $self = shift;
+    return $self->{cfg}{LogDeclines} ||
+            $self->{apache}->dir_config('AxLogDeclines') ||
+            0;
+}
+
+sub HandleDirs {
+    my $self = shift;
+    return $self->{cfg}{HandleDirs} ||
+            $self->{apache}->dir_config('AxHandleDirs') ||
+            0;
+}
+
+sub IgnoreStylePI {
+    my $self = shift;
+    return $self->{cfg}{IgnoreStylePI} ||
+            $self->{apache}->dir_config('AxIgnoreStylePI') ||
+            0;
+}
+
+sub AllowOutputCharset {
+    my $self = shift;
+    
+    my $oldval = $self->{output_charset_ok};
+    if (@_) {
+        $self->{output_charset_ok} = shift;
+    }
+    return $oldval;
+}
+    
 sub OutputCharset {
     my $self = shift;
+
+    return unless $self->{output_charset_ok};
+    
+#    warn "OutputCharset\n";
+    unless ($self->{cfg}{TranslateOutput} ||
+            $self->{apache}->dir_config('AxTranslateOutput')) {
+        return;
+    }
+    
+#    warn "Checking OutputCharset\n";
     
     if (my $charset = $self->{cfg}{OutputCharset}
         || $self->{apache}->dir_config('AxOutputCharset')) {
         return $charset;
     }
     
+#    warn "Checking Accept-Charset\n";
     # check HTTP_ACCEPT_CHARSET
     if (my $ok_charsets = $ENV{HTTP_ACCEPT_CHARSET}) {
         my @charsets = split(/,\s*/, $ok_charsets);
@@ -123,17 +199,19 @@ sub OutputCharset {
 
             ($charset, $score) = split(/;\s*q=/, $charset, 2);
             $score = 1 unless (defined($score) && ($score =~ /^(\+|\-)?\d+(\.\d+)?$/));
-            
-            if ($score > $retscore || $charset =~ /^utf\-?8$/) { # we like utf8
+           
+            if ($score > $retscore || $charset =~ /^utf\-?8$/i) { # we like utf8
                 $retcharset = $charset;
                 $retscore = $score;
             }
         }
         
         $retcharset =~ s/iso/ISO/;
+        $retcharset =~ s/(us\-)?ascii/US-ASCII/;
         
         return undef if $retcharset =~ /^utf\-?8$/;
-
+	return undef if $retcharset eq '*';
+# warn "Charset: '$retcharset'\n";
         return $retcharset;
     }
     
@@ -146,7 +224,7 @@ sub ErrorStyles {
     my ($type, $href);
     if (!$style) {
         ($type, $href) = split(/\s*=>\s*/,
-                $self->{apache}->dir_config('AxErrorStylesheet'),
+                ($self->{apache}->dir_config('AxErrorStylesheet') || ''),
                 2);
         return [] unless $href;
     }
@@ -158,6 +236,12 @@ sub ErrorStyles {
     
     my $module = $style_map->{ $type };
     
+    if (!$module) {
+        throw Apache::AxKit::Exception::Error(
+                -text => "ErrorStylesheet: No module mapping found for type '$type'"
+                );
+    }
+    
     return [{href => $href, type => $type, module => $module}];
 }
 
@@ -213,30 +297,53 @@ sub DoGzip {
 
 sub GetMatchingProcessors {
     my $self = shift;
-    my ($media, $style, $doctype, $dtd, $root) = @_;
+    my ($media, $style, $doctype, $dtd, $root, $styles, $provider) = @_;
+    return @$styles if @$styles;
     
+    $style ||= '#default';
+
     my $list = $self->{cfg}{Processors}{$media}{$style};
+
+    my $processors = $self->{apache}->dir_config('AxProcessors');
+    if( $processors ) {
+      foreach my $processor (split(/\s*,\s*/, $processors) ) {
+	my ($pmedia, $pstyle, @processor) = split(/\s+/, $processor);
+	next unless ($pmedia eq $media and $pstyle eq $style);
+	push (@$list, [ 'NORMAL', @processor ] );
+      }
+    }
     
     my @results;
     
     for my $directive (@$list) {
         my $type = $directive->[0];
+        my $style_hash = {
+                    type => $directive->[1], 
+                    href => $directive->[2],
+                    title => $style,
+                };
         if ($type eq 'NORMAL') {
-            push @results, { type => $directive->[1], href => $directive->[2] };
+            push @results, $style_hash;
         }
         elsif ($type eq 'DocType') {
             if ($doctype eq $directive->[3]) {
-                push @results, { type => $directive->[1], href => $directive->[2] };
+                push @results, $style_hash;
             }
         }
         elsif ($type eq 'DTD') {
             if ($dtd eq $directive->[3]) {
-                push @results, { type => $directive->[1], href => $directive->[2] };
+                push @results, $style_hash;
             }
         }
         elsif ($type eq 'Root') {
             if ($root eq $directive->[3]) {
-                push @results, { type => $directive->[1], href => $directive->[2] };
+                push @results, $style_hash;
+            }
+        }
+        elsif ($type eq 'URI') {
+            my $uri = $provider->apache_request->uri;
+            if ($uri =~ /$directive->[3]/) {
+                push @results, $style_hash;
             }
         }
         else {
@@ -244,6 +351,16 @@ sub GetMatchingProcessors {
         }
     }
     
+    # list any dynamically chosen stylesheets here
+    $list = $self->{cfg}{DynamicProcessors};
+    foreach my $package (@$list) {
+        AxKit::load_module($package);
+        no strict 'refs';
+        my($handler) = $package.'::handler';
+        push @results, $handler->($provider, $media, $style, 
+                                  $doctype, $dtd, $root);
+    }   
+    
     return @results;
 }
 
@@ -253,8 +370,11 @@ sub XSPTaglibs {
     my @others;
     
     @others = eval { keys %{ $self->{cfg}{XSPTaglibs} } };
+    warn $@ if $@;
     
-    return @others if @others;
+    if (@others) {
+        return @others;
+    }
     
     local $^W;
     @others = split(/\s+/, $self->{apache}->dir_config('AxAddXSPTaglibs'));
@@ -262,4 +382,34 @@ sub XSPTaglibs {
     return @others;
 }
 
+sub OutputTransformers {
+    my $self = shift;
+
+    my @filters;
+
+    if( my $o_t = $self->{cfg}{OutputTransformers} ) {
+        @filters = @$o_t;
+    }
+    else {
+        @filters = split(/\s+/, 
+                $self->{apache}->dir_config('AxOutputTransformers'));    
+    }
+
+    return @filters;
+}
+
+sub Plugins {
+    my $self = shift;
+    my @plugs;
+
+    if (my $plugins = $self->{cfg}{Plugins}) {
+        @plugs = @$plugins;
+    }
+    else {
+        @plugs = split(/\s+/,
+                $self->{apache}->dir_config('AxPlugins'));
+    }
+    return @plugs;
+}
+
 1;
