@@ -1,4 +1,4 @@
-# $Id: XPathScript.pm,v 1.33 2000/10/01 22:08:16 matt Exp $
+# $Id: XPathScript.pm,v 1.3 2002/03/25 14:52:20 jwalt Exp $
 
 package Apache::AxKit::Language::XPathScript;
 
@@ -7,7 +7,7 @@ use vars qw(@ISA $VERSION $stash );
 
 use Apache;
 use Apache::File;
-use XML::XPath 0.50;
+use XML::XPath 1.00;
 use XML::XPath::XMLParser;
 use XML::XPath::Node;
 use XML::XPath::NodeSet;
@@ -15,11 +15,8 @@ use XML::Parser;
 use Apache::AxKit::Provider;
 use Apache::AxKit::Language;
 use Apache::AxKit::Cache;
-use Apache::AxKit::Exception ':try';
-use Storable;
-use Unicode::Map8 ();
-use Unicode::String ();
-
+use Apache::AxKit::Exception;
+use Apache::AxKit::CharsetConv;
 
 @ISA = 'Apache::AxKit::Language';
 
@@ -27,7 +24,7 @@ $VERSION = '0.05';
 
 sub handler {
     my $class = shift;
-    my ($r, $xml_provider, $style_provider, $reparse) = @_;
+    my ($r, $xml_provider, $style_provider) = @_;
     
     my $xpath = XML::XPath->new();
     
@@ -35,63 +32,40 @@ sub handler {
     
     my $xml_parser = XML::Parser->new(
             ErrorContext => 2,
-            Namespaces => 1,
+            Namespaces => $XML::XPath::VERSION < 1.07 ? 1 : 0,
             ParseParamEnt => 1,
             );
     
     my $parser = XML::XPath::XMLParser->new(parser => $xml_parser);
     
-    local *XML::XPath::Function::document;
+    local $Apache::AxKit::Language::XPathScript::local_ent_handler;
     
     if (my $entity_handler = $xml_provider->get_ext_ent_handler()) {
+#        warn "XPathScript: setting entity_handler\n";
         $xml_parser->setHandlers(
                 ExternEnt => $entity_handler,
                 );
-        setup_document_function($entity_handler, $parser);
+        $Apache::AxKit::Language::XPathScript::local_ent_handler = $entity_handler;
     }
     
     AxKit::Debug(6, "XPathScript: Getting XML Source");
     
     if (my $dom = $r->pnotes('dom_tree')) {
-        use XML::XPath::Builder;
-        my $builder = XML::XPath::Builder->new();
-        $source_tree = $dom->to_sax( Handler => $builder );
-        $dom->dispose;
+        # dom_tree is an XML::LibXML DOM
+        $source_tree = $parser->parse($dom->toString);
         delete $r->pnotes()->{'dom_tree'};
     }
-    elsif (my $xml = $r->notes('xml_string')) {
+    elsif (my $xml = $r->pnotes('xml_string')) {
 #        warn "Parsing from string : $xml\n";
-        $source_tree = $parser->parse($xml);
+        eval {
+            $source_tree = $parser->parse($xml);
+        };
+        if ($@) {
+            throw Apache::AxKit::Exception::Error(-text => "Parse of xml_string failed: $@");
+        }
     }
     else {
-        AxKit::Debug(6, "XPathScript: creating stor cache");
-        my $cache = Apache::AxKit::Cache->new($r, $r->filename(), 'xpathstor');
-        my $key = $xml_provider->key();
-        if (!$reparse) {
-            AxKit::Debug(7, "XPathScript: thawing stor cache");
-            try {
-                $source_tree = Storable::thaw($cache->read());
-            }
-            catch Error with {
-                my $err = shift;
-                AxKit::Debug(7, "XPathScript: thaw failed: $err");
-            };
-        }
-        
-        if (!$source_tree) {
-            AxKit::Debug(7, "XPathScript: reparsing file");
-            $source_tree = try {
-                $parser->parse($xml_provider->get_fh());
-            }
-            catch Error with {
-                $parser->parse(${ $xml_provider->get_strref() });
-            };
-            
-            AxKit::Debug(7, "XPathScript: freezing stor cache");
-
-            $cache->write(Storable::freeze($source_tree));
-        }
-        
+        $source_tree = get_source_tree($xml_provider, $parser);
     }
     
     $xpath->set_context($source_tree);
@@ -104,7 +78,7 @@ sub handler {
     AxKit::Debug(6, "Checking stylesheet mtime: $mtime\n");
     if ($stash->{$style_key}
             && exists($stash->{$style_key}{mtime})
-            && ($stash->{$style_key}{mtime} <= $mtime)
+            && !$style_provider->has_changed($stash->{$style_key}{mtime})
             && check_inc_mtime($stash->{$style_key}{mtime}, $style_provider, $stash->{$style_key}{includes})) {
         # cached... just exec.
         AxKit::Debug(7, "Using stylesheet cache\n");
@@ -115,31 +89,61 @@ sub handler {
         compile($package, $style_provider);
         $stash->{$style_key}{mtime} = get_mtime($class, $style_provider);
     }
-    
-    my $old_status = $r->status;
-    
+
     no strict 'refs';
     my $cv = \&{"$package\::handler"};
 
-    $Apache::AxKit::Language::XPathScript::xp = $xpath;
+    local $Apache::AxKit::Language::XPathScript::xp = $xpath;
     my $t = {};
-    $Apache::AxKit::Language::XPathScript::trans = $t;
-    
+    local $Apache::AxKit::Language::XPathScript::trans = $t;
+    local $Apache::AxKit::Language::XPathScript::style_provider = $style_provider;
+
     AxKit::Debug(7, "Running XPathScript script\n");
     local $^W;
-    $cv->($r, $xpath, $t);
-        
-    if (!$r->notes('xml_string')) { # no output? Try apply_templates
+    my $rc = Apache::Constants::OK;
+    eval {
+        $rc = $cv->($r, $xpath, $t);
+    };
+    if ($@) {
+        AxKit::Debug(1, "XPathScript error: $@");
+        throw $@;
+    }
+
+    if (!$r->pnotes('xml_string') &&
+        !$r->dir_config('XPSNoApplyTemplatesOnEmptyOutput')) { # no output? Try apply_templates
         print Apache::AxKit::Language::XPathScript::Toys::apply_templates();
     }
-    
+
 #    warn "Run\n";
 
     $Apache::AxKit::Language::XPathScript::xp = undef;
     $Apache::AxKit::Language::XPathScript::trans = undef;
-    
+    $Apache::AxKit::Language::XPathScript::style_provider = undef;
 #    warn "Returning $old_status\n";
-    return $r->status($old_status);
+    return $rc;
+}
+
+sub get_source_tree {
+    my ($provider, $parser) = @_;
+    my $source_tree;
+    AxKit::Debug(7, "XPathScript: reparsing file");
+    eval {
+        my $fh = $provider->get_fh();
+        # warn("parsing FH $fh with parser $parser\n");
+        local $/;
+        my $contents = <$fh>;
+        # warn("FH contains: $contents\n");
+        $source_tree = $parser->parse($contents);
+        # warn("Parse completed\n");
+        close($fh);
+    };
+    if ($@) {
+        # warn("parse_fh failed\n");
+        $source_tree = $parser->parse(${ $provider->get_strref() });
+    }
+
+    # warn("get_source_tree = $source_tree\n");
+    return $source_tree;
 }
 
 sub check_inc_mtime {
@@ -148,11 +152,11 @@ sub check_inc_mtime {
     for my $inc (@$includes) {
 #        warn "Checking mtime for $inc\n";
         my $inc_provider = Apache::AxKit::Provider->new(
-                Apache->request, 
+                AxKit::Apache->request, 
                 uri => $inc,
                 rel => $provider,
                 );
-        if ($inc_provider->mtime() < $mtime) {
+        if ($inc_provider->has_changed($mtime)) {
 #            warn "$inc newer (" . $inc_provider->mtime() . ") than last compile ($mtime) causing recompile\n";
             return;
         }
@@ -161,24 +165,25 @@ sub check_inc_mtime {
 }
 
 sub extract {
-    my ($provider) = @_;
+    my ($provider,$scalar_output) = @_;
     
-    my $contents = try { 
+    my $contents;
+    eval { 
         my $fh = $provider->get_fh();
         local $/;
-        return <$fh>;
-    }
-    catch Error with {
-        return ${ $provider->get_strref() };
+        $contents = <$fh>;
     };
+    if ($@) {
+        $contents = ${ $provider->get_strref() };
+    }
     
-    my $r = $provider->apache_request;
+    my $r = AxKit::Apache->request();
     if (my $charset = $r->dir_config('XPathScriptCharset')) {
         
         AxKit::Debug(8, "XPS: got charset: $charset");
         
-        my $map = Unicode::Map8->new($charset) || die "No such charset: $charset";
-        $contents = $map->tou($contents)->utf8();
+        my $map = Apache::AxKit::CharsetConv->new($charset, "utf-8") || die "No such charset: $charset";
+        $contents = $map->convert($contents);
     }
     
     my $key = $provider->key();
@@ -194,12 +199,20 @@ sub extract {
         my ($text, $type) = ($1, $2);
         $line += $text =~ tr/\n//;
         $text =~ s/\|/\\\|/g;
-        $script .= "print q|$text|;";
+        if($scalar_output) {
+            $script .= "\$__OUTPUT.=q|$text|;";
+        } else {
+            $script .= "print q|$text|;";
+        }
         $script .= "\n#line $line $key\n";
         if ($type eq '<%=') {
             $contents =~ /\G(.*?)%>/gcs || die "No terminating '%>' after line $line ($key)";
             my $perl = $1;
-            $script .= "print( $perl );\n";
+            if(!$scalar_output) {
+                $script .= "print(do { $perl });\n";
+            } else {
+                $script .= "\$__OUTPUT.=join('',(do { $perl }));\n";
+            }
             $line += $perl =~ tr/\n//;
         }
         elsif ($type eq '<!--#include') {
@@ -214,7 +227,7 @@ sub extract {
             }
             
             AxKit::Debug(10, "About to include file $params{file}");
-            $script .= include_file($params{file}, $provider);
+            $script .= include_file($params{file}, $provider, $scalar_output);
             AxKit::Debug(10, "include done");
         }
         else {
@@ -229,7 +242,11 @@ sub extract {
     if ($contents =~ /\G(.*)/gcs) {
         my ($text) = ($1);
         $text =~ s/\|/\\\|/g;
-        $script .= "print q|$text|;";
+        if ($scalar_output) {
+            $script .= "\$__OUTPUT.=q|$text|;";
+        } else {
+            $script .= "print q|$text|;";
+        }
     }
     
     return $script;
@@ -250,58 +267,87 @@ sub compile {
             'my ($r, $xp, $t) = @_;',
             "\n#line 1 " . $provider->key() . "\n",
             $script,
+            ";\n",
+            'return Apache::Constants::OK;',
             "\n}",
             );
-    
+
     local $^W;
-    
+
     AxKit::Debug(10, "Compiling script:\n$eval\n");
     eval $eval;
-    throw Apache::AxKit::Exception::Error(-text => "$@") if $@;
+    if ($@) {
+        AxKit::Debug(1, "Compilation failed: $@");
+        throw $@;
+    }
 }
 
 sub include_file {
-    my ($filename, $provider) = @_;
+    my ($filename, $provider, $script_output) = @_;
 
     # return if already included
     my $key = $provider->key();
     return '' if grep {$_ eq $filename} @{$stash->{$key}{includes}};
-    
+
     push @{$stash->{$key}{includes}}, $filename;
     
     my $inc_provider = Apache::AxKit::Provider->new(
-            Apache->request,
+            AxKit::Apache->request,
             uri => $filename,
             rel => $provider,
             );
     
-    return extract($inc_provider);
+    return extract($inc_provider, $script_output);
 }
 
-sub setup_document_function {
-    my ($ent_handler, $parser) = @_;
-    no strict 'refs';
-    undef *{'XML::XPath::Function::document'};
-    *{'XML::XPath::Function::document'} = sub {
-        my $self = shift;
-        my ($node, @params) = @_;
-        die "document: Function takes 1 parameter\n" unless @params == 1;
-        my $results = XML::XPath::NodeSet->new();
-        my $newdoc;
-        try {
-            $newdoc = $parser->parse($ent_handler->(undef, undef, $params[0]));
-        }
-        catch Apache::AxKit::Exception::IO with {
-            my $E = shift;
-            AxKit::Debug(2, $E);
-        }
-        catch Error with {
-            my $E = shift;
-            throw Apache::AxKit::Exception::Error(-text => "Parse of '$params[0]' failed: $E");
+sub XML::XPath::Function::document {
+    # warn "Document function called\n";
+    return unless $Apache::AxKit::Language::XPathScript::local_ent_handler;
+    my $self = shift;
+    my ($node, @params) = @_;
+    die "document: Function takes 1 parameter\n" unless @params == 1;
+
+    my $xml_parser = XML::Parser->new(
+            ErrorContext => 2,
+            Namespaces => $XML::XPath::VERSION < 1.07 ? 1 : 0,
+            # ParseParamEnt => 1,
+            );
+
+    my $parser = XML::XPath::XMLParser->new(parser => $xml_parser);
+
+    my $results = XML::XPath::NodeSet->new();
+    my $uri = $params[0];
+    my $newdoc;
+    if ($uri =~ /^\w\w+:/) { # assume it's scheme://foo uri
+        eval {
+            # warn "Trying to parse $params[0]\n";
+            $newdoc = $parser->parse(
+                    $Apache::AxKit::Language::XPathScript::local_ent_handler->(
+                        undef, undef, $uri
+                    )
+                );
+            # warn "Parsed OK into $newdoc\n";
         };
-        $results->push($newdoc) if $newdoc;
-        return $results;
-    };
+        if (my $E = $@) {
+            if ($E->isa('Apache::AxKit::Exception::IO')) {
+                AxKit::Debug(2, $E);
+            }
+            else {
+                throw Apache::AxKit::Exception::Error(-text => "Parse of '$uri' failed: $E");
+            };
+        }
+    }
+    else {
+        # warn("Parsing local: $uri\n");
+        my $provider = Apache::AxKit::Provider->new(
+                    AxKit::Apache->request, 
+                    uri => $uri,
+                );
+        $newdoc = get_source_tree($provider, $parser);
+    }
+
+    $results->push($newdoc) if $newdoc;
+    return $results;
 }
 
 sub get_mtime {
@@ -321,13 +367,13 @@ sub get_mtime {
 
     for my $inc (@{$stash->{$filename}{includes}}) {
         my $inc_provider = Apache::AxKit::Provider->new(
-                Apache->request, 
+                AxKit::Apache->request, 
                 uri => $inc,
                 rel => $provider,
                 );
         
 #        warn "Checking mtime of $inc\n";
-        if ($inc_provider->mtime() < $mtime) {
+        if ($inc_provider->has_changed($mtime)) {
             $mtime = $inc_provider->mtime();
         }
     }
@@ -356,6 +402,7 @@ sub get_package_name {
     package Apache::AxKit::Language::XPathScript::Toys;
     
     use XML::XPath::Node;
+    use Apache::AxKit::Exception;
 
     use vars '@ISA', '@EXPORT';
     use Exporter;
@@ -368,7 +415,40 @@ sub get_package_name {
             apply_templates
             matches
             set_namespace
+            import_template
+            DO_SELF_AND_KIDS
+            DO_SELF_ONLY
+            DO_NOT_PROCESS
             );
+    
+    sub DO_SELF_AND_KIDS () { return 1; }
+    sub DO_SELF_ONLY () { return -1; }
+    sub DO_NOT_PROCESS () { return 0; }
+    sub MAX_DEPTH () { return 32; }
+    
+    sub import_template {
+        my ($filename, $local_changes) = @_;
+        my ($script) = Apache::AxKit::Language::XPathScript::include_file($filename,$Apache::AxKit::Language::XPathScript::style_provider, 1);
+        # changes may be local to this imported template, or global (default).
+        my ($setup_t);
+        if ($local_changes) {
+            $setup_t = 'local $Apache::AxKit::Language::XPathScript::trans = clone($Apache::AxKit::Language::XPathScript::trans);';
+        }
+        
+        $script = join('',
+                     'use strict;',
+                     'sub { ',
+                     'my ($node, $real_local_t) = @_;',
+                     'local $Apache::AxKit::Language::XPathScript::xp = $node;',
+                     $setup_t,
+                     'my ($t) = $Apache::AxKit::Language::XPathScript::trans;',
+                     'my ($__OUTPUT);',
+                     $script,';',
+                     '$real_local_t->{pre} = $__OUTPUT;',
+                     'return -1;',
+                     '}');
+        return eval($script);
+    }
 
     sub findnodes {
         $Apache::AxKit::Language::XPathScript::xp->findnodes(@_);
@@ -390,17 +470,19 @@ sub get_package_name {
     sub matches {
         $Apache::AxKit::Language::XPathScript::xp->matches(@_);
     }
-    
+
     sub set_namespace {
-        try {
+        eval {
             $Apache::AxKit::Language::XPathScript::xp->set_namespace(@_);
-        }
-        catch Error with {
-            my $E = shift;
-            warn "set_namespace failed: $E";
         };
+        if ($@) {
+            AxKit::Debug(3, "set_namespace failed: $@");
+        }
     }
     
+    # quieten warnings when compiling this module
+    sub apply_templates (;$@);
+    
     sub apply_templates (;$@) {
         unless (@_) {
             return apply_templates(findnodes('/'));
@@ -411,16 +493,24 @@ sub get_package_name {
         if (!ref($arg1)) {
             # called with a path to find
 #            warn "apply_templates with path '$arg1'\n";
-            return apply_templates(findnodes($arg1, @args));
+            $arg1 = findnodes($arg1, @args);
+#            return apply_templates($nodes);
         }
         
         my $retval = '';
-        if ($arg1->isa('XML::XPath::NodeSet')) {
+        if (ref($arg1) eq "HASH") {
+#            warn "apply_templates with a hash\n";
+            local $Apache::AxKit::Language::XPathScript::trans = $arg1;
+            return apply_templates(@args);
+        } 
+        elsif ($arg1->isa('XML::XPath::NodeSet')) {
+#            warn "apply_templates with a NodeSet\n";
             foreach my $node ($arg1->get_nodelist) {
                 $retval .= translate_node($node);
             }
         }
         else {
+#            warn "apply_templates with a list of " , 1 + @args, " nodes? : ", ref($arg1), "\n";
             $retval .= translate_node($arg1);
             foreach my $node (@args) {
                 $retval .= translate_node($node);
@@ -440,7 +530,7 @@ sub get_package_name {
         
         return $retval;
     }
-    
+
     sub translate_node {
         my $node = shift;
         
@@ -448,13 +538,32 @@ sub get_package_name {
                 
         my $translations = $Apache::AxKit::Language::XPathScript::trans;
         
+        if ($node->isTextNode) {
+            my $trans = $translations->{'text()'};
+            if (!$trans) { return $node->toString; }
+            if (my $code = $trans->{testcode}) {
+                my $t = {};
+                my $retval = $code->($node, $t);
+                if ($retval && %$t) {
+                    foreach my $tkey (keys %$t) {
+                        $trans->{$tkey} = $t->{$tkey};
+                    }
+                }
+            }
+            return $trans->{pre} . $node->toString . $trans->{post};
+        }
+
         if (!$node->isElementNode) {
             # don't output top-level PI's
             if ($node->isPINode) {
-                if (try {$node->getParentNode->getParentNode}) {
-                    return $node->toString;
-                }
-                return '';
+                my $retstring = eval {
+                    if ($node->getParentNode->getParentNode) {
+                        return $node->toString;
+                    }
+                    return '';
+                };
+                
+                return $retstring || '';
             }
             return $node->toString;
         }
@@ -462,6 +571,7 @@ sub get_package_name {
 #        warn "translate_node: ", $node->getName, "\n";
         
         my $node_name = $node->getName;
+
         my $trans = $translations->{$node_name};
 
         if (!$trans) {
@@ -470,35 +580,63 @@ sub get_package_name {
         }
         
         if (!$trans) {
-            return start_tag($node) . 
-                    _apply_templates($node->getChildNodes) .
+#            warn "Default trans\n";
+            if (my @children = $node->getChildNodes) {
+                return start_tag($node) . 
+                    _apply_templates(@children) .
                     end_tag($node);
+            }
+            else {
+                return empty_tag($node);
+            }
         }
-        
+
         local $^W;
         
         my $dokids = 1;
         my $search;
-
+       my $testcode_output = 0;
         my $t = {};
         if ($trans->{testcode}) {
-#            warn "Evalling testcode\n";
-            my $result = $trans->{testcode}->($node, $t);
-            if ($result eq "0") {
-                # don't process anything.
-                return;
+#            warn "eval testcode\n";
+           $testcode_output = 1;
+            my $result;
+            my $testcode = $trans->{testcode};
+            my $depth = 0;
+            while (1) {
+                $result = $testcode->($node, $t);
+#                warn "Testcode returned: $result\n";
+                if (defined($t->{testcode}) &&
+                      ref($t->{testcode}) eq "CODE") {
+                    if ($depth++ > MAX_DEPTH) {
+                        die "Max Depth of ", MAX_DEPTH, " reached on testcode eval!";
+                    }
+                    $testcode = $t->{testcode};
+                    $t = {};
+                } else {
+                    last;
+                }
             }
-            if ($result eq "-1") {
-                # -1 means don't do children.
-                $dokids = 0;
-            }
-            elsif ($result eq "1") {
-                # do kids
-            }
-            else {
+            
+#            warn "Here with $result\n";
+            
+
+            if ($result =~ /\D/) {
                 $dokids = 0;
                 $search = $result;
             }
+            elsif ($result == DO_NOT_PROCESS) {
+                # don't process anything.
+                return;
+            }
+            elsif ($result == DO_SELF_ONLY) {
+                # -1 means don't do children.
+                $dokids = 0;
+            }
+            elsif ($result == DO_SELF_AND_KIDS) {
+                # do kids
+            }
+#            warn "Here with dokids => $dokids, search => $search\n";
         }
         
         local $translations->{$node_name};
@@ -513,21 +651,21 @@ sub get_package_name {
         }
         
         # default: process children too.
-        my $pre = $trans->{pre} . 
+        my $pre = interpolate($node, $trans->{pre}, $testcode_output) . 
                 ($trans->{showtag} ? start_tag($node) : '') .
-                $trans->{prechildren};
+                interpolate($node, $trans->{prechildren}, $testcode_output);
         
-        my $post = $trans->{postchildren} .
+        my $post = interpolate($node, $trans->{postchildren}, $testcode_output) .
                 ($trans->{showtag} ? end_tag($node) : '') .
-                $trans->{post};
-        
+                interpolate($node, $trans->{post}, $testcode_output);
+
         if ($dokids) {
             my $middle = '';
             for my $kid ($node->getChildNodes()) {
                 if ($kid->isElementNode) {
-                    $middle .= $trans->{prechild} .
+                    $middle .= interpolate($node, $trans->{prechild}) .
                             _apply_templates($kid) .
-                            $trans->{postchild};
+                            interpolate($node, $trans->{postchild});
                 }
                 else {
                     $middle .= _apply_templates($kid);
@@ -539,9 +677,9 @@ sub get_package_name {
             my $middle = '';
             for my $kid (findnodes($search, $node)) {
                 if ($kid->isElementNode) {
-                    $middle .= $trans->{prechild} .
+                    $middle .= interpolate($node, $trans->{prechild}) .
                             _apply_templates($kid) .
-                            $trans->{postchild};
+                            interpolate($node, $trans->{postchild});
                 }
                 else {
                     $middle .= _apply_templates($kid);
@@ -585,6 +723,60 @@ sub get_package_name {
             return '';
         }
     }
+    
+    sub empty_tag {
+        my ($node) = @_;
+        
+        my $name = $node->getName;
+        return '' unless $name;
+        
+        my $string = "<" . $name;
+        
+        foreach my $ns ($node->getNamespaceNodes) {
+            $string .= $ns->toString;
+        }
+        
+        foreach my $attr ($node->getAttributeNodes) {
+            $string .= $attr->toString;
+        }
+
+        $string .= " />";
+        
+        return $string;
+    }        
+    
+    sub interpolate {
+        my ($node, $string, $ignore) = @_;
+        return $string if $XPathScript::DoNotInterpolate || $ignore;
+        return $string unless AxKit::Apache->request->dir_config('AxXPSInterpolate');
+        my $new = '';
+        while ($string =~ m/\G(.*?)\{(.+?)\}/gcs) {
+            my ($pre, $path) = ($1, $2);
+            $new .= $pre;
+            $new .= $node->findvalue($path);
+        }
+        $string =~ /\G(.*)/gcs;
+        $new .= $1 if defined $1;
+        return $new;
+    }
+
+    # make a clone, but copy subs.
+    sub clone {
+        my ($a) = @_;
+        my ($b);
+        if (ref($a) eq "HASH") {
+            $b = {};
+            foreach my $key (keys(%$a)) {        
+                my ($copy) = clone($a->{$key});
+                $b->{$key} = $copy;
+            }
+        }
+        else {
+            # copy as is
+            $b = $a;
+        }
+        return $b;
+    }
 
     1;
 }
@@ -604,7 +796,7 @@ Apache::AxKit::Language::XPathScript - An XML Stylesheet Language
 =head1 DESCRIPTION
 
 This documentation has been removed. The definitive reference for 
-XPathScript is now at http://xml.sergeant.org/axkit/xpathscript/guide.dkb
+XPathScript is now at http://axkit.org/docs/xpathscript/guide.dkb
 in DocBook format.
 
 =cut
