@@ -1,9 +1,11 @@
-# $Id: Provider.pm,v 1.13 2000/09/24 15:10:55 matt Exp $
+# $Id: Provider.pm,v 1.4 2002/04/02 16:27:54 matts Exp $
 
 package Apache::AxKit::Provider;
 use strict;
 
-use Apache::AxKit::Exception ':try';
+use Apache::AxKit::Exception;
+use Apache::Constants qw(OK DECLINED);
+#use XML::Parser;
 
 # use vars qw/$COUNT/;
 
@@ -18,8 +20,8 @@ sub new {
     
     $self->init(@_);
     
-#     AxKit::Debug(7, "Provider->new Count: " . ++$COUNT);
-    
+    AxKit::add_depends($self->key());
+
     return $self;
 }
 
@@ -36,24 +38,56 @@ sub apache_request {
     return $self->{apache};
 }
 
+sub has_changed {
+    my $self = shift;
+    my $time = shift;
+    return $self->mtime > $time;
+}
+
+sub decline {
+    my $self = shift;
+
+    AxKit::Debug(4, "provider declined");
+    return DECLINED;
+}
+
 sub get_ext_ent_handler {
     my $self = shift;
     return sub {
         my ($e, $base, $sysid, $pubid) = @_;
-        if ($sysid =~ /^(https?|ftp):/) {
+#        warn "ext_ent: base => $base, sys => $sysid, pub => $pubid\n";
+        if ($sysid =~ /^http:/) {
             if ($pubid) {
                 return ''; # do not bring in public DTD's
             }
-            return XML::Parser::lwp_ext_ent_handler(@_);
+            eval {
+                require HTTP::GHTTP;
+            };
+            if ($@) {
+                require LWP::Simple;
+                import LWP::Simple;
+                return get($sysid) || die "Cannot get $sysid";
+            }
+            my $r = HTTP::GHTTP->new($sysid);
+            $r->process_request;
+            return $r->get_body;
         }
-        
+        elsif ($sysid =~ /^(https|ftp):/) {
+            if ($pubid) {
+                return ''; # do not bring in public DTD's
+            }
+            die "Cannot download https (SSL) or ftp URL's yet. Patches welcome";
+        }
+
 #        warn "File provider ext_ent_handler called with '$sysid'\n";
-        $sysid =~ s/^file:(\/\/)?//;
         my $provider = Apache::AxKit::Provider->new(
-                Apache->request,
-                uri => $sysid
+                AxKit::Apache->request,
+                uri => $sysid,
+#                rel => $self,
                 );
+#        warn "Got provider with key: ", $provider->key, "\n";
         my $str = $provider->get_strref;
+#        warn "Returning string with length: ", length($$str), "\n";
         return $$str;
     };
 }
@@ -61,80 +95,79 @@ sub get_ext_ent_handler {
 sub get_styles {
     my $self = shift;
     my ($media, $pref_style) = @_;
-    
+
     if ($pref_style eq '#default') {
         undef $pref_style;
     }
-    
-    my $styles = [];
-    my $ext_ents = [];
+
+    my $xml_styles = [];
     my $vals = [];
-    
+
     my $key = $self->key();
-    
+
     # need to extract the following from the XML file:
     #   DocType Public Identifier
     #   DTD filename
     #   Root element name (including namespace)
     # use three element array @$vals
-    
-    AxKit::Debug(4, "get_styles: creating XML::Parser");
-    
-    my $xml_parser = XML::Parser->new(
-                ParseParamEnt => 1, 
-                ErrorContext => 2,
+
+    if (defined &Apache::AxKit::Provider::xs_get_styles_fh) {
+        AxKit::Debug(2, "using XS get_styles (libxml2)");
+        my ($xs_styles, $doctype, $dtd, $root) =
+                $self->xs_get_styles($media, $pref_style);
+        @$xml_styles = @$xs_styles unless $AxKit::Cfg->IgnoreStylePI();
+        @$vals = ($doctype, $dtd, $root);
+    }
+    else {
+        require XML::Parser;
+
+        AxKit::Debug(4, "get_styles: creating XML::Parser");
+
+        my $handlers = {
+                    Start => \&parse_start,
+                    Doctype => \&parse_dtd,
+                    $AxKit::Cfg->IgnoreStylePI() ? () : (Proc => \&parse_pi),
+                    };
+
+        my $xml_parser = XML::Parser->new(
                 Namespaces => 1,
-                );
-    $xml_parser->setHandlers(
-                Start => \&parse_start,
-                Doctype => \&parse_dtd,
-                Proc => \&parse_pi,
-                Entity => \&parse_entity_decl,
-                );
-    
-    my $to_parse = try { 
-        $self->get_fh();
-    } catch Error with {
-        ${ $self->get_strref(); };
-    };
-    
-    try {
+                ErrorContext => 2,
+                Handlers => $handlers,
+            );
+
+        my $to_parse;
+        eval {
+            $to_parse = $self->get_fh();
+        };
+        if ($@) {
+            $to_parse = ${ $self->get_strref(); };
+        }
+        
         AxKit::Debug(4, "get_styles: calling XML::Parser->parse('$key')");
         $xml_parser->parse(
                 $to_parse,
                 XMLStyle_preferred => $pref_style,
-                XMLStyle_style => $styles,
-                XMLStyle_ext_ents => $ext_ents,
+                XMLStyle_media => $media,
+                XMLStyle_style => $xml_styles,
                 XMLStyle_vals => $vals,
                 XMLStyle_style_screen => [],
-                XMLStyle_media => $media,
                 );
-    }
-    catch Error with {
-        my $E = shift;
-        AxKit::Debug(4, "get_styles: parse returned: >> $E <<");
-        if ("$E" !~ /^OK/) { # note that regex bindings don't stringify in 5.6.0 - fixed in 5.6.1
-            throw Apache::AxKit::Exception::Error(
-                    -text => "Parsing '$key' returned: $E\n"
-                    );
-        }
-    };
-        
-    AxKit::Debug(4, "get_styles: parse returned successfully");
-
-    my @styles;
-        
-    if (@$styles) {
-        @styles = @$styles;
-    }
-    else {
-        # now get all current styles that match all these properties
-        @styles = $AxKit::Cfg->GetMatchingProcessors($media, $pref_style, @$vals);
+            
+        AxKit::Debug(4, "get_styles: parse returned successfully");
     }
     
+    foreach my $style (@$xml_styles) {
+        $style->{title} ||= '#default';
+    }
+    
+    # Let GetMatchingProcessors to process the @$styles array
+    AxKit::Debug(4, "Calling GetMatchingProcessors with ($media, $pref_style, $vals->[0], $vals->[1], $vals->[2])");
+    my @styles = $AxKit::Cfg->GetMatchingProcessors($media,
+		$pref_style, @$vals[0 .. 2], $xml_styles, $self);
+    
     if (!@styles) {
-        throw Apache::AxKit::Exception::Error(
-                -text => "No styles defined for '$key'"
+        throw Apache::AxKit::Exception::Declined(
+                reason => "No styles defined for '$key'"
                 );
     }
     
@@ -154,18 +187,75 @@ sub get_styles {
         $style->{module} = $mapto;
 
         # first load module if it's not already loaded.
-        try {
+        eval {
             AxKit::load_module($mapto);
-        }
-        catch Error with {
-            my $E = shift;
-            throw Apache::AxKit::Exception::Declined(
-                    reason => "Load of '$mapto' failed with: $E"
-                    );
         };
+        if ($@) {
+            throw Apache::AxKit::Exception::Error(
+                    -text => "Load of '$mapto' failed with: $@"
+                    );
+        }
     }
     
-    return \@styles, $ext_ents;
+    return \@styles;
+}
+
+sub xs_get_styles {
+    my $self = shift;
+    my ($media, $pref_style) = @_;
+    
+    my $bits;
+    eval {
+        my $fh = $self->get_fh();
+        AxKit::Debug(4, "calling xs_get_styles_fh()");
+        $bits = xs_get_styles_fh($self->apache_request, $fh);
+    };
+    if ($@) {
+        my $strref = $self->get_strref();
+        AxKit::Debug(4, "calling xs_get_styles_str()");
+        $bits = xs_get_styles_str($self->apache_request, $$strref);
+    }
+    
+    my @xml_stylesheet = @{$bits->[0]};
+    my %attribs = @{$bits->[2]};
+    my $element = $bits->[1];
+    
+    # resolve namespaces (libxml doesn't!)
+    if ($attribs{'xmlns'} && $element !~ /:/) {
+        $element = "{$attribs{xmlns}}$element";
+    }
+    elsif ($element =~ /^(.*):(.*)$/) {
+        my ($prefix, $el) = ($1, $2);
+        my $ns = $attribs{"xmlns:$prefix"};
+        $element = "{$ns}$el";
+    }
+    
+    my $e = {
+        XMLStyle_style => [], 
+        XMLStyle_style_screen => [],
+        XMLStyle_preferred => $pref_style,
+        XMLStyle_media => $media,
+        };
+    
+    foreach my $pi (@xml_stylesheet) {
+        parse_pi($e, "xml-stylesheet", $pi);
+    }
+    
+    if (!@{$e->{XMLStyle_style}} && !$e->{XMLStyle_style_persistant}) {
+        if ($e->{XMLStyle_style_screen_persistant}) {
+            push @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_screen_persistant}};
+        }
+        if (@{$e->{XMLStyle_style_screen}}) {
+            push @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_screen}};
+        }
+    }
+    elsif ($e->{XMLStyle_style_persistant}) {
+        unshift @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_persistant}};
+    }
+    
+    AxKit::Debug(4, "xs_get_styles returned: $bits->[3], $bits->[4], $element");
+    
+    return ($e->{XMLStyle_style}, $bits->[3], $bits->[4], $element);
 }
 
 sub parse_start {
@@ -174,14 +264,11 @@ sub parse_start {
     # use James Clark's universal name format
     $e->{XMLStyle_vals}[2] = $ns ? "{$ns}$el" : $el;
     
-#    warn "styles: ", scalar @{$e->{XMLStyle_style_screen_persistant}}, "\n";
-    
     if (!@{$e->{XMLStyle_style}} && !$e->{XMLStyle_style_persistant}) {
         if ($e->{XMLStyle_style_screen_persistant}) {
             push @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_screen_persistant}};
         }
         if (@{$e->{XMLStyle_style_screen}}) {
-    #        warn "Matching style for media ", $e->{XMLStyle_media}, " not found. Using screen media stylesheets instead\n";
             push @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_screen}};
         }
     }
@@ -189,17 +276,7 @@ sub parse_start {
         unshift @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_persistant}};
     }
     
-    die "OK\n";
-}
-
-sub parse_entity_decl {
-    my $e = shift;
-    my ($name, $val, $sysid, $pubid, $ndata) = @_;
-#    warn "external entity: '$sysid'\n";
-    if (!defined $val) {
-        # external entity - save so the cache gets done properly!
-        push @{$e->{XMLStyle_ext_ents}}, $sysid;
-    }
+    $e->finish();
 }
 
 sub parse_dtd {
@@ -365,11 +442,9 @@ Return the last modification time in days before the current time.
 
 =head2 get_styles()
 
-Extract the stylesheets and external entities from the XML resource. Should
-return a list of ($styles, $ext_ents). Both are array refs, the style
-entries are hashes refs with required keys 'href' and 'type'. The external
-entities entries are scalars containing the system identifier of the 
-external entity.
+Extract the stylesheets from the XML resource. Should return an array
+ref of styles. The style entries are hashes refs with required keys
+'href' and 'type'.
 
 =head2 get_fh()
 
