@@ -1,9 +1,9 @@
-# $Id: FileSuffix.pm,v 1.5 2000/06/12 16:51:50 matt Exp $
+# $Id: FileSuffix.pm,v 1.2 2002/02/01 14:45:07 matts Exp $
 
 package Apache::AxKit::StyleChooser::FileSuffix;
 
 use strict;
-use Apache::Constants;
+use Apache::Constants qw(OK);
 use Apache::URI;
 
 sub handler {
@@ -17,7 +17,7 @@ sub handler {
 #		warn "setting notes: $style\n";
 		$r->notes('preferred_style', $style);
 	}
-	return DECLINED;
+	return OK;
 }
 
 1;
@@ -27,11 +27,16 @@ __END__
 
 Apache::AxKit::StyleChooser::FileSuffix - Choose stylesheet using file suffix
 
+=head1 SYNOPSIS
+
+	PerlTypeHandler Apache::AxKit::StyleChooser::FileSuffix
+	SetHandler perl-script
+	PerlHandler AxKit
+
 =head1 DESCRIPTION
 
 This module lets you pick a stylesheet based on the filename suffix. To use
-it, simply add this module to the list of PerlTypeHandlers prior to
-Apache::AxKit::XMLFinder:
+it, simply add this module to the list of PerlTypeHandlers:
 
 	PerlTypeHandler Apache::AxKit::StyleChooser::FileSuffix
 	SetHandler perl-script
