@@ -1,9 +1,9 @@
-# $Id: PathInfo.pm,v 1.4 2000/06/12 16:20:51 matt Exp $
+# $Id: PathInfo.pm,v 1.2 2002/02/01 14:45:07 matts Exp $
 
 package Apache::AxKit::StyleChooser::PathInfo;
 
 use strict;
-use Apache::Constants;
+use Apache::Constants qw(OK);
 use Apache::URI;
 
 sub handler {
@@ -29,7 +29,7 @@ sub handler {
 #		warn "setting notes: $style\n";
 		$r->notes('preferred_style', $style);
 	}
-	return DECLINED;
+	return OK;
 }
 
 1;
@@ -39,11 +39,16 @@ __END__
 
 Apache::AxKit::StyleChooser::PathInfo - Choose stylesheet using PATH_INFO
 
+=head1 SYNOPSIS
+
+	PerlHandler Apache::AxKit::StyleChooser::PathInfo \
+			AxKit
+
 =head1 DESCRIPTION
 
 This module lets you pick a stylesheet based on the extra PATH_INFO. To use
 it, simply add this module to the list of PerlHandlers prior to
-Apache::XMLStylesheet:
+AxKit:
 
 	PerlHandler Apache::AxKit::StyleChooser::PathInfo \
 			AxKit
